#include <Alert.h>
#include <string.h>

#include "Berc.h"
#include "ConfigMessage.h"

#include "liblirc.h"
#include "RcDefs.h"
#include "BercDefs.h"

int32 bercd_thread( void* device )
{
   return(((BercDevice*)device)->Thread());
}

//Instance new device
BInputServerDevice* instantiate_input_device()
{
   return(new BercDevice());
}

//EasyPen device constructor
BercDevice::BercDevice( void ):BInputServerDevice()
{
   int i_ret;
   BAlert         *pc_AlertMessage;
   
   input_device_ref dev = { "Be Remote Control", B_KEYBOARD_DEVICE, NULL };
   input_device_ref *dev_list[2] = { &dev, NULL };

   RegisterDevices( dev_list );
   rc_keymap = gas_keymap;
   pc_PortQueue = new BercPortQueue();
}

//EasyPen destructor
BercDevice::~BercDevice(void)
{
   delete pc_PortQueue;
}

//Initialization check
status_t BercDevice::InitCheck()
{
   return(B_OK);
}

status_t BercDevice::Start( const char *device, void * )
{   
   char  sz_Module[255], sz_Driver[255];
   status_t i_ret;
   ConfigMessage  *pc_Config;
   BAlert         *pc_AlertMessage;

   pc_Config = new ConfigMessage(BERC_CONFIG_FILE);
   if(pc_Config == NULL)
   {
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: Unable to read config!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
      return(B_ERROR);
   }

   if(pc_Config->GetModule(sz_Module) != B_OK)
   {
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: No module!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
      return(B_ERROR);
   }
      
   if(pc_Config->GetDriver(sz_Driver) != B_OK)
   {
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: No driver!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
      return(B_ERROR);
   }
     
   if(lirc_init(sz_Module, sz_Driver) != 0)
   {
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: lirc_init error!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
   }
   
   if(pc_Config->GetFirstRemote(sz_Driver) != B_OK)
   {
      lirc_deinit();
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: No remote!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
      return(B_ERROR);
   }
   
   if(lirc_add_remote(sz_Driver) != 0)
   {
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: lirc_add_remote!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
   }
   
   if(lirc_init_device() != 0)
   {
      pc_AlertMessage = 
         new BAlert("Berc InputServerDevice","BercDevice::Start: lirc_init_device!",
         "OK",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
      pc_AlertMessage->Go(NULL);
   }
   
   b_Quit = false;
   threadid = spawn_thread(bercd_thread, device, B_DISPLAY_PRIORITY, this );
   resume_thread( threadid );

   delete pc_Config;
   return(B_NO_ERROR);
}

status_t BercDevice::Stop( const char *, void * )
{
   b_Quit = true;
   status_t err = B_OK;
   wait_for_thread( threadid, &err );
   lirc_deinit();
   return(B_NO_ERROR);
}
      
status_t  BercDevice::Control(const char *name, void *cookie,
                              uint32 command, BMessage *message)
{   
   char *psz_Port;
   uint32 i_Filter;
   
   switch(message->what)
   {
      case RC_REGISTER_CLIENT:
         if(message->FindString("berc:port", (const char**)&psz_Port) == B_OK)
         {
            if(message->FindInt32("berc:filter", (int32*)&i_Filter) == B_OK)
               pc_PortQueue ->AddPort(psz_Port, i_Filter);
         }
         break;

      case RC_UNREGISTER_CLIENT:
         /* nothing to do: The port client will be automatically deleted when
            sending the next keycode                                          */
         break;
   }
}

//Main Thread
int32 BercDevice::Thread(void)
{
   char  *message;
   char  sz_KeyString[32];
   
   while(!b_Quit )
   {
      message = lirc_read_data();
      if(message != NULL)
      {
         char *p_StartPos;
         char *p_CurPos = message;
         
         /* get only the key string */
         while( (*p_CurPos != ' ') && (*p_CurPos != '\0') )
            p_CurPos++;
         if(*p_CurPos != '\0')
            p_CurPos++;
         
         while( (*p_CurPos != ' ') && (*p_CurPos != '\0') )
            p_CurPos++;
         if(*p_CurPos != '\0')
            p_CurPos++;
         
         p_StartPos = p_CurPos;
         while( (*p_CurPos != ' ') && (*p_CurPos != '\0') )
            p_CurPos++;
         if(p_CurPos - p_StartPos < 32)
         {
            strncpy(sz_KeyString, p_StartPos, p_CurPos - p_StartPos);
            sz_KeyString[p_CurPos - p_StartPos] = '\0';
         }
        
         /* then send the key message */
         SendKeyMessage(sz_KeyString);
      }
   }
   return(B_OK);
}


TS_RC_keymap *BercDevice::GetKeyMessageParams(char *sz_Keyname)
{
   TS_RC_keymap   *ps_CurKey;

   ps_CurKey = rc_keymap;

   while(ps_CurKey->rc_keyname != NULL)
   {
      if(strcmp(sz_Keyname, ps_CurKey->rc_keyname) == 0)
         return(ps_CurKey);
      ps_CurKey++;
   }
   return(NULL);
}

int32 BercDevice::SendKeyMessage(char *sz_Keyname)
{
   char  utf_key[3];
   int32 key_offset, key_size;
   BMessage* event;
   bigtime_t TimeNow;
   TS_RC_keymap   *ps_KeyMessage;
   
   //Get time now
   TimeNow = system_time();

   ps_KeyMessage = GetKeyMessageParams(sz_Keyname);
   
   
   if(ps_KeyMessage != NULL)
   {
      event = new BMessage( ps_KeyMessage->message );
      if(event == NULL)
         return(B_ERROR);

      /* common fields of messages */
      event->AddBool("berc:key", true);
      event->AddInt64( "when", TimeNow);
      get_key_map(&system_keymap, &system_char);
  
      /* mapped key fields */
      if(ps_KeyMessage->message == B_KEY_DOWN)
      {
         //make the rest of the message
         event->AddInt32("key", ps_KeyMessage->keycode);

         if(ps_KeyMessage->modifiers_mask & B_SHIFT_KEY)
            key_offset = system_keymap->shift_map[ps_KeyMessage->keycode];
         else
            key_offset = system_keymap->normal_map[ps_KeyMessage->keycode];

         key_size = system_char[key_offset++];
         if(key_size < 4)
         {
            strncpy(utf_key, &(system_char[key_offset]), key_size);
            event->AddData("byte", B_INT8_TYPE, utf_key, key_size);
         }
         event->AddInt32("modifiers", modifiers());
      }
      else if(ps_KeyMessage->message == B_MODIFIERS_CHANGED)
      {
         int32 i_CurModifiers;
         key_info s_KeyInfo;
         
         if(get_key_info(&s_KeyInfo) != B_OK)
            return(B_ERROR);

         i_CurModifiers = s_KeyInfo.modifiers;         
         event->AddInt32("be:old_modifiers", i_CurModifiers);
         event->AddInt32("modifiers", i_CurModifiers | B_NUM_LOCK);
         event->AddData("states", B_UINT8_TYPE, s_KeyInfo.key_states, 16);
      }
      else  /* unmapped key fields */
      {
         event->AddInt32("key", ps_KeyMessage->keycode);
      }

      /* Message is sent to the port if the port filter match. If SendMessage
         returns an error it means that either the port was deleted, or that the
         filter does not match. In both cases we send the message through the
         input sever.                                                         */
      if(pc_PortQueue->SendMessage(event) != B_OK)
         EnqueueMessage( event );
   }
   else  /* custom key message */
   {
      uint16   i_KeyMask;
      event = new BMessage(B_UNMAPPED_KEY_DOWN);
      if(event == NULL)
         return(B_ERROR);

      /* common fields of messages */
      event->AddBool("berc:key", true);
      event->AddInt64( "when", TimeNow);

      i_KeyMask = ((uint16)sz_Keyname[strlen(sz_Keyname)-2]) << 8;
      i_KeyMask |= (uint16)(sz_Keyname[strlen(sz_Keyname)-1]);
      
      event->AddInt32("modifiers", B_NUM_LOCK);
      event->AddInt32("key", RC_UNMAPPED_KEY_CUSTOM | i_KeyMask);
      event->AddString("berc:custom_key", sz_Keyname);
      
      if(pc_PortQueue->SendMessage(event) != B_OK)
         EnqueueMessage( event );
   }
   
   return(B_OK);
}
