
/*
*****************************************************************************
*
*                          Copyright (c) 2000 2004
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice, 
*     this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice, 
*     this list of conditions and the following disclaimer in the documentation
*     and/or other materials provided with the distribution.
*  - The names of its contributors may not be used to endorse or promote 
*     products derived from this software without specific prior written 
*     permission.
*   
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************
* File          : BercPort.cpp
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description :
******************************************************************************
*/

/* ########################################################################## */
/* ### #include                                                               */
#include <stdlib.h>

#include "BercPort.h"
#include "RcDefs.h"

/* ########################################################################## */
/* ### Variables                                                              */

/* ########################################################################## */
/* ### Functions                                                              */

/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : BercPort
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
*  constructor
******************************************************************************
*/
BercPort::BercPort(char *sz_PortName, uint32 i_PortFilter)
{
   i_Port = find_port(sz_PortName);
   i_Filter = i_PortFilter;
   pc_Next = NULL;
}


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : ~BercPort
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
*  Destructor
******************************************************************************
*/
BercPort::~BercPort()
{
   return ;
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : FilterMatch
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
bool BercPort::FilterMatch(BMessage *pc_Message)
{
   int32 i_KeyMsg;
   status_t i_ret;
   
   switch(pc_Message->what)
   {
      case B_KEY_DOWN:
         if(i_Filter & RC_KEY_MAPPED)
            return(true);
         break;

      case B_UNMAPPED_KEY_DOWN:
         i_ret = pc_Message->FindInt32((const char*)"key", (int32*)&i_KeyMsg);         
         if(i_ret != B_OK)
            return(false);

         if(i_KeyMsg == RC_UNMAPPED_KEY_CUSTOM)
         {
            if(i_Filter & RC_KEY_CUSTOM)
               return(true);
         }
         else if( (i_KeyMsg == RC_KEY_POWER) ||
                  (i_KeyMsg == RC_KEY_MUTE) ||
                  (i_KeyMsg == RC_KEY_VOLUME_UP) ||
                  (i_KeyMsg == RC_KEY_VOLUME_DOWN) )
         {
            if(i_Filter & RC_KEY_SYSTEM)
               return(true);
         }
         else if(i_Filter & RC_KEY_UNMAPPED)
            return(true);
                  
         break;
   }
   return(false);
} 

/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : SendMessage
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
*  Write a BMessage into the port.
******************************************************************************
*/
status_t BercPort::SendMessage(BMessage *pc_Message)
{
   char    *p_MessageData;
   uint32   i_Size;
   status_t i_ret;
   
   if(i_Port == B_NAME_NOT_FOUND)
      return(B_BAD_PORT_ID);
  
   i_Size = pc_Message->FlattenedSize();
   
   /* write header */
   i_ret = write_port(i_Port,  RC_PORT_HEADER_CODE, &i_Size, sizeof(uint32));
   if(i_ret != B_OK)
      return(i_ret);
   
   /* write data */
   p_MessageData = (char*) malloc(i_Size);
   if(p_MessageData == NULL)
      return(B_ERROR);
   
   pc_Message->Flatten(p_MessageData, i_Size);
   i_ret = write_port(i_Port,  RC_PORT_MESSAGE_CODE, p_MessageData, i_Size);

   if(i_ret != B_OK)
   {
      free(p_MessageData);
      return(i_ret);
   }
   
   free(p_MessageData);
   return(B_OK);
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : GetNext
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
BercPort *BercPort::GetNext(void)
{   
   return(pc_Next);
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : SetNext
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
status_t BercPort::SetNext(BercPort *pc_Port)
{
   pc_Next = pc_Port;
   return(B_OK);
} 
