
/*
*****************************************************************************
*
*                          Copyright (c) 2000 2004
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice, 
*     this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice, 
*     this list of conditions and the following disclaimer in the documentation
*     and/or other materials provided with the distribution.
*  - The names of its contributors may not be used to endorse or promote 
*     products derived from this software without specific prior written 
*     permission.
*   
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************
* File          : BercPortQueue.cpp
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description :
******************************************************************************
*/

/* ########################################################################## */
/* ### #include                                                               */
#include "BercPortQueue.h"

/* ########################################################################## */
/* ### Variables                                                              */

/* ########################################################################## */
/* ### Functions                                                              */


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : BercPortQueue
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
*  Constructor.
******************************************************************************
*/
BercPortQueue::BercPortQueue(void)
{
   pc_ListEntry = NULL;
   return ;
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : ~BercPortQueue
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
*  Queue destructor. The port chainlist is fred here.
******************************************************************************
*/
BercPortQueue::~BercPortQueue(void)
{
   BercPort *pc_Port, *pc_NewPort;

   /* free the chainlist */
   pc_Port = pc_ListEntry;
   while(pc_Port != NULL)
   {
      pc_NewPort = pc_Port->GetNext();
      delete(pc_Port);
      pc_Port = pc_NewPort;
   }
   return ;
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : AddPort
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
*  Adds a new port in the head of the port chainlist.
******************************************************************************
*/
status_t BercPortQueue::AddPort(char *sz_Port, uint32 i_Filter)
{
   BercPort *pc_NewPort;

   pc_NewPort = new BercPort(sz_Port, i_Filter);
   if(pc_NewPort == NULL)
      return(B_ERROR);
   
   if(pc_ListEntry != NULL)
      pc_NewPort->SetNext(pc_ListEntry);
   pc_ListEntry = pc_NewPort;
   
   return(B_OK);
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : SendMessage
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
status_t BercPortQueue::SendMessage(BMessage *pc_Message)
{
   status_t i_ret;
   BercPort *pc_Port;

   if(pc_ListEntry == NULL)
      return(B_ERROR);

   if(pc_ListEntry->FilterMatch(pc_Message) == false)
      return(B_ERROR);
   
   i_ret = pc_ListEntry->SendMessage(pc_Message);
   while( (i_ret != B_OK) && (pc_ListEntry != NULL) )
   {
      /* an error means that the port was deleted by the client. Thus we remove
       it from the chainlist.                                                 */
      pc_Port = pc_ListEntry;
      pc_ListEntry = pc_Port->GetNext();
      delete(pc_Port);
      
      if(pc_ListEntry != NULL)
      {
         if(pc_ListEntry->FilterMatch(pc_Message) == false)
            return(B_ERROR);
         i_ret = pc_ListEntry->SendMessage(pc_Message);
      }
   }
   return(i_ret);
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : IsEmpty
* Author        : Zadig
* Creation date : 14/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
bool BercPortQueue::IsEmpty(void)
{
   if(pc_ListEntry == NULL)
      return(true);
   else
      return(false);
} 
