#include <stdio.h>

#include "ConfigMessage.h"

ConfigMessage::ConfigMessage(char *psz_File) : BMessage('RcCf')
{
   pc_ConfigFile = new BFile(psz_File, B_READ_WRITE | B_CREATE_FILE);
   Unflatten(pc_ConfigFile);
   pc_ConfigFile->Seek(0,SEEK_SET);
}

ConfigMessage::~ConfigMessage()
{
   delete pc_ConfigFile;
}

status_t ConfigMessage::Flush(void)
{   
   uint16   i_Version;
   ssize_t  i_Size;
   status_t i_ret;
   
   if(FindInt16((const char*)CONFIG_VERSION_FIELD, (int16*)&i_Version) != B_OK)
      AddInt16(CONFIG_VERSION_FIELD, CONFIG_VERSION);
   
   i_Size = FlattenedSize();
   i_ret = Flatten(pc_ConfigFile, &i_Size);
   pc_ConfigFile->Seek(0,SEEK_SET);
   return(i_ret);
}

status_t ConfigMessage::GetModule(char *psz_Module)
{
   char *psz_LocalModule;
   if(FindString(CONFIG_MODULE_FIELD, (const char**)&psz_LocalModule) == B_OK)
   {
      strcpy(psz_Module, psz_LocalModule);
   }
   else
   {
      psz_Module[0] = '\0';
      return(B_ENTRY_NOT_FOUND);
   }
   return(B_OK);
}

status_t ConfigMessage::SetModule(char *psz_Module)
{
   char *psz_LocalModule;
   if(FindString(CONFIG_MODULE_FIELD, (const char**)&psz_LocalModule) == B_OK)
      return(ReplaceString(CONFIG_MODULE_FIELD, psz_Module));
   else
      return(AddString(CONFIG_MODULE_FIELD, psz_Module));
}

status_t ConfigMessage::GetDriver(char *psz_Driver)
{
   char *psz_LocalDriver;
   if(FindString(CONFIG_DRIVER_FIELD, (const char**)&psz_LocalDriver) == B_OK)
   {
      strcpy(psz_Driver, psz_LocalDriver);
   }
   else
   {
      psz_Driver[0] = '\0';
      return(B_ENTRY_NOT_FOUND);
   }
   return(B_OK);
}

status_t ConfigMessage::SetDriver(char *psz_Driver)
{
   char *psz_LocalDriver;
   if(FindString(CONFIG_DRIVER_FIELD, (const char**)&psz_LocalDriver) == B_OK)
      return(ReplaceString(CONFIG_DRIVER_FIELD, psz_Driver));
   else
      return(AddString(CONFIG_DRIVER_FIELD, psz_Driver));
}

status_t ConfigMessage::GetFirstRemote(char *psz_Remote)
{
   i_RemoteIndex = 0;
   return(GetNextRemote(psz_Remote));
}

status_t ConfigMessage::GetNextRemote(char *psz_Remote)
{
   char *psz_LocalRemote;
   if(FindString(CONFIG_REMOTE_FIELD, i_RemoteIndex, 
                  (const char**)&psz_LocalRemote) == B_OK)
   {
      strcpy(psz_Remote, psz_LocalRemote);
      i_RemoteIndex++;
      return(B_OK);
   }

   return(B_ENTRY_NOT_FOUND);
}

status_t ConfigMessage::AddRemote(char *psz_Remote)
{
   uint8 i_CurIndex = 0;
   char *psz_LocalRemote;
   
   /* check that the remote is not already in the message */
   while(FindString(CONFIG_REMOTE_FIELD, i_CurIndex, 
         (const char**)&psz_LocalRemote) == B_OK)
   {
      if(strcmp(psz_Remote, psz_LocalRemote) == 0)
         return(B_OK);  /* remote already in message */
      i_CurIndex++;
   }
      
   return(AddString(CONFIG_REMOTE_FIELD, psz_Remote));
}

status_t ConfigMessage::DelRemote(char *psz_Remote)
{
   uint8 i_CurIndex = 0;
   char *psz_LocalRemote;
   
   /* find the remote is in the message */
   while(FindString(CONFIG_REMOTE_FIELD, i_CurIndex, 
         (const char**)&psz_LocalRemote) == B_OK)
   {
      if(strcmp(psz_Remote, psz_LocalRemote) == 0)
         return(RemoveData(CONFIG_REMOTE_FIELD, i_CurIndex));  /* and delete */
      i_CurIndex++;
   }
      
   return(B_ENTRY_NOT_FOUND);
}

