#include <Window.h>
#include <Application.h>
#include <Button.h>
#include <Directory.h>
#include <Input.h>
#include <TextControl.h>
#include <Alert.h>
#include <String.h>
#include <View.h>
#include <GraphicsDefs.h>
#include <Path.h>
#include <Roster.h>
#include <Screen.h>


#include <stdio.h>

#include "BercDefs.h"
#include "Preference.h"
#include "PrefMessages.h"
#include "ModuleGui.h"

Preference gc_MainApp;

int main()
{
   gc_MainApp.Run();
   return 0;
}

Preference::Preference():BApplication("application/x-vnd.zadig.BercPreference")
{
   BScreen  *pc_Screen = new BScreen();
   BRect c_WindowPosition(100, 100, 100+W_WIDTH, 100+W_HEIGHT);

   if(pc_Screen != NULL)
   {
      c_WindowPosition = pc_Screen->Frame();
      c_WindowPosition.left = c_WindowPosition.right - c_WindowPosition.left;
      c_WindowPosition.left /= 2;
      c_WindowPosition.left -= (W_WIDTH/2);
      c_WindowPosition.right = c_WindowPosition.left + W_WIDTH;

      c_WindowPosition.top = c_WindowPosition.bottom - c_WindowPosition.top;
      c_WindowPosition.top /= 2;
      c_WindowPosition.top -= (W_HEIGHT/2);
      c_WindowPosition.bottom = c_WindowPosition.top + W_HEIGHT;
   }
   
   pc_MainWindow = new MainWindow(c_WindowPosition);
}

void Preference::MessageReceived(BMessage *message)
{
   switch (message->what)
   {
      case PREF_MSG_REMOTE_OPEN:
         pc_MainWindow->RemoteAddRequest(message);
         break;
   }
}

MainWindow::MainWindow(BRect r)  :
   BWindow(r, "Berc Device Preferences", B_TITLED_WINDOW_LOOK, 
            B_NORMAL_WINDOW_FEEL, 
            B_ASYNCHRONOUS_CONTROLS | B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
   char  sz_Module[MODULE_STRING_SIZE], sz_Driver[MODULE_STRING_SIZE];
   char  sz_Remote[REMOTE_STRING_SIZE];
   float i_Width, i_Height;
   BRect   c_ButtonView;
   BInputDevice   *pc_BercDevice;
   
   pc_MainView = new BView(Bounds(), "MainView",          
         B_FOLLOW_LEFT,B_NAVIGABLE);
   pc_MainView->SetViewColor(BG_COLOR);
   AddChild(pc_MainView);

   pc_ModuleGui = new ModuleGui(pc_MainView, Bounds().top);
   pc_RemoteGui = new RemoteGui(pc_MainView, Bounds().top + 80);

   c_ButtonView = Bounds();
   c_ButtonView.top += 250;
   c_ButtonView.left += 5;
   pc_StopButton = new BButton(c_ButtonView,"Switch Daemon", "Stop  Daemon",
         new BMessage(PREF_MSG_SWITCH_DAEMON));
   pc_StopButton->GetPreferredSize(&i_Width,&i_Height);
   pc_StopButton->ResizeTo(i_Width,i_Height);
   
   pc_BercDevice = find_input_device("Be Remote Control");
   if(pc_BercDevice == NULL)
   {
      pc_StopButton->SetLabel("No Daemon!");
   }
   else
   {
      if(pc_BercDevice->IsRunning() == false)
         pc_StopButton->SetLabel("Start Daemon");
   }

   pc_MainView->AddChild(pc_StopButton);

   pc_ApplyButton = new BButton(c_ButtonView,"Apply Changes", "Apply",
         new BMessage(PREF_MSG_APPLY));
   pc_ApplyButton->GetPreferredSize(&i_Width,&i_Height);
   pc_ApplyButton->ResizeTo(i_Width,i_Height);
   pc_ApplyButton->MoveTo(pc_StopButton->Frame().right + 10,c_ButtonView.top);
   pc_MainView->AddChild(pc_ApplyButton);

   SetDefaultButton(pc_ApplyButton);

   /* load settings */
   BDirectory  c_ConfigDir;
   c_ConfigDir.CreateDirectory(BERC_CONFIG_PATH, NULL);
   pc_ConfigMessage = new ConfigMessage(BERC_CONFIG_FILE);
   pc_ConfigMessage->GetModule(sz_Module);
   pc_ConfigMessage->GetDriver(sz_Driver);
   pc_ModuleGui->SetModule(sz_Module, sz_Driver);
        
   if(pc_ConfigMessage->GetFirstRemote(sz_Remote) == B_OK)
   {
      pc_RemoteGui->AddRemote(sz_Remote);
      while(pc_ConfigMessage->GetNextRemote(sz_Remote) == B_OK)
         pc_RemoteGui->AddRemote(sz_Remote);
   }

   /* write default settings if none where selected */
   if(pc_ConfigMessage->GetModule(sz_Module) != B_OK)
   {
      pc_ModuleGui->GetModule(sz_Module, sz_Driver);
      pc_ConfigMessage->SetModule(sz_Module);
      pc_ConfigMessage->SetDriver(sz_Driver);
      pc_ConfigMessage->Flush();
   }

   Show();
}

MainWindow::~MainWindow()
{
}

void MainWindow::MessageReceived(BMessage *message)
{
   char     sz_Module[255], sz_Driver[255];
   char     sz_DelRemote[REMOTE_STRING_SIZE];
   
   switch (message->what)
   {
      case PREF_MSG_APPLY:
         pc_ConfigMessage->Flush();
         RestartDaemon();
         break;

      case PREF_MSG_SWITCH_DAEMON:
         SwitchDaemonState();
         break;

      case PREF_MSG_MODULE_CHANGED:
         Lock();
         pc_ModuleGui->UpdateDriverList();
         pc_ModuleGui->GetModule(sz_Module, sz_Driver);
         pc_ConfigMessage->SetModule(sz_Module);
         pc_ConfigMessage->SetDriver(sz_Driver);
         Unlock();
         break;

      case PREF_MSG_DRIVER_CHANGED:
         pc_ModuleGui->GetModule(sz_Module, sz_Driver);
         pc_ConfigMessage->SetDriver(sz_Driver);
         break;

      case PREF_MSG_ADD_REMOTE:
         pc_RemoteGui->AddRemote("");
         break;

      case PREF_MSG_DEL_REMOTE:
         if(pc_RemoteGui->DelRemote(sz_DelRemote) == B_OK)
            pc_ConfigMessage->DelRemote(sz_DelRemote);
         break;
   }
}

bool MainWindow::QuitRequested()
{
   be_app->PostMessage(B_QUIT_REQUESTED);
   return true;
}

status_t MainWindow::RemoteAddRequest(BMessage *message)
{
   char        sz_RemoteFile[REMOTE_STRING_SIZE];
   int32       i_Index;
   ssize_t     i_Size;
   entry_ref   s_Entry;
   
   Lock();
   i_Index = 0;
   while(message->FindRef("refs",i_Index, &s_Entry) == B_OK)
   {
      BEntry   c_Entry(&s_Entry, true);
      BPath    c_Path;

      c_Entry.GetPath(&c_Path);
      pc_RemoteGui->AddRemote((char*)c_Path.Path());
      pc_ConfigMessage->AddRemote((char*)c_Path.Path());
      i_Index++;
   }
   Unlock();
}

status_t MainWindow::RestartDaemon(void)
{
   bool     b_BercState;
   BMessage *pc_Msg;
   BInputDevice   *pc_BercDevice;
   
   pc_BercDevice = find_input_device("Be Remote Control");
   if(pc_BercDevice == NULL)
      return(B_OK);

   if(pc_BercDevice->IsRunning() == true)
   {
      pc_BercDevice->Stop();
      pc_BercDevice->Start();
   }
   return(B_OK);
}
      
status_t MainWindow::SwitchDaemonState(void)
{
   bool     b_BercState;
   BMessage *pc_Msg;
   BInputDevice   *pc_BercDevice;
   
   pc_BercDevice = find_input_device("Be Remote Control");
   if(pc_BercDevice == NULL)
      return(B_OK);

   if(pc_BercDevice->IsRunning() == true) /* we must stop the server */
   {
      pc_BercDevice->Stop();
      pc_StopButton->SetLabel("Start Daemon");
   }
   else  /* we must start the server */
   {
      pc_BercDevice->Start();
      pc_StopButton->SetLabel("Stop Daemon");
   }
   return(B_OK);
}

