
#include <Alert.h>
#include <BitmapStream.h>
#include <Message.h>
#include <Roster.h>
#include <TypeConstants.h>
#include <TranslationUtils.h>
#include <TranslatorRoster.h>

#include <string.h>

#include "VolumeBar.h"

VolumeBar::VolumeBar()
{
   uint8 i,j;
   BRect c_Rect(20, 20, 41, 335);
   BAlert *ErrorAlert;

   /* create the window */
   pc_Window = new BWindow(c_Rect, "Volume", B_NO_BORDER_WINDOW_LOOK,
         B_FLOATING_ALL_WINDOW_FEEL, B_AVOID_FOCUS);
   if(pc_Window == NULL)
      return;

   /* read attributes */

   /* create all views */
   if(InitBitmaps() == B_OK)
   {
      for(i=0; i< NB_VOLUME_LEVEL; i++)
      {
         c_Rect = pc_Window->Bounds();
         c_Rect.top = c_Rect.top + 21*i;
         c_Rect.bottom = c_Rect.top + 21 * (i+1);

         pac_Views[i] = new BView(c_Rect, "VolumeView", 
               B_FOLLOW_LEFT,B_NAVIGABLE);
         if(pac_Views[i] == NULL)
         {
            ErrorAlert = new BAlert("Berc Filter Error", "Error while creating view", 
                  "OK",NULL,NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
            ErrorAlert->Go(NULL);

            for(j=0; j<i; j++)
               delete pac_Views[j];
            return;
         }
         pc_Window->AddChild(pac_Views[i]);
         pac_Views[i]->SetViewBitmap(pac_Bitmaps[BITMAP_GREY]);
      }
   }
   pc_Window->Show();
   pc_Window->Hide();
}

VolumeBar::~VolumeBar()
{
   if(pc_Window != NULL)
      pc_Window->Quit();
}

status_t VolumeBar::SetVolumeLevel(uint8 i_Level)
{
   uint8 i;
  
   pc_Window->Lock();
   for(i=0; i< NB_VOLUME_LEVEL; i++)
   {
      if(i_Level > i)
      {
         if(i>=10)
            pac_Views[NB_VOLUME_LEVEL-1-i]->SetViewBitmap(pac_Bitmaps[BITMAP_RED]);
         else if(i>=5)
            pac_Views[NB_VOLUME_LEVEL-1-i]->SetViewBitmap(pac_Bitmaps[BITMAP_GREEN]);
         else
            pac_Views[NB_VOLUME_LEVEL-1-i]->SetViewBitmap(pac_Bitmaps[BITMAP_BLUE]);
      }
      else
         pac_Views[NB_VOLUME_LEVEL-1-i]->SetViewBitmap(pac_Bitmaps[BITMAP_GREY]);
   }
   pc_Window->Unlock();
   
   pc_Window->Show();
   snooze(1000000);
   pc_Window->Hide();
}

status_t VolumeBar::InitBitmaps()
{
   int32    i_Id;
   size_t   i_Size;
   BFile f_File;
   void  *p_Data;
   BMessage *pc_BitmapMessage;
   BAlert *ErrorAlert;
   translator_info info;
   BBitmapStream outstream1, outstream2, outstream3, outstream4;
   
   BTranslatorRoster *proster = BTranslatorRoster::Default();
   if (!proster)
      return(B_ERROR);

   /* load grey image */
   if(f_File.SetTo(IMAGES_FILES "/grey.png", B_READ_ONLY) != 
         B_OK)
      return(B_ERROR);
   
   memset(&info, 0, sizeof(translator_info));
   if (proster->Identify(&f_File, NULL, &info, 0, NULL,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);

   if (proster->Translate(&f_File, &info, NULL, &outstream1,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);
   if (outstream1.DetachBitmap(&(pac_Bitmaps[BITMAP_GREY])) != B_OK)
      return(B_ERROR);

   /* load blue image */
   if(f_File.SetTo(IMAGES_FILES "/blue.png", B_READ_ONLY) != 
         B_OK)
      return(B_ERROR);
   
   memset(&info, 0, sizeof(translator_info));
   if (proster->Identify(&f_File, NULL, &info, 0, NULL,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);

   if (proster->Translate(&f_File, &info, NULL, &outstream2,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);
   if (outstream2.DetachBitmap(&(pac_Bitmaps[BITMAP_BLUE])) != B_OK)
      return(B_ERROR);
   
   /* load green image */
   if(f_File.SetTo(IMAGES_FILES "/green.png", B_READ_ONLY) != 
         B_OK)
      return(B_ERROR);
   
   memset(&info, 0, sizeof(translator_info));
   if (proster->Identify(&f_File, NULL, &info, 0, NULL,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);

   if (proster->Translate(&f_File, &info, NULL, &outstream3,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);
   if (outstream3.DetachBitmap(&(pac_Bitmaps[BITMAP_GREEN])) != B_OK)
      return(B_ERROR);

   /* load red image */
   if(f_File.SetTo(IMAGES_FILES "/red.png", B_READ_ONLY) != 
         B_OK)
      return(B_ERROR);
   
   memset(&info, 0, sizeof(translator_info));
   if (proster->Identify(&f_File, NULL, &info, 0, NULL,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);

   if (proster->Translate(&f_File, &info, NULL, &outstream4,
            B_TRANSLATOR_BITMAP) != B_OK)
      return(B_ERROR);
   if (outstream4.DetachBitmap(&(pac_Bitmaps[BITMAP_RED])) != B_OK)
      return(B_ERROR);
   
   return(B_OK);
}
