#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <drivers/config_manager.h>

#include "serial.h"

static config_manager_for_driver_module_info *gps_ConfigManager;

status_t init_cm_wrapper(void)
{
   if (get_module(B_CONFIG_MANAGER_FOR_DRIVER_MODULE_NAME,
            (module_info **)&gps_ConfigManager) < 0)
   {      
      return B_ERROR;
   }
   return(B_OK);
}

status_t uninit_cm_wrapper(void)
{
   put_module(B_CONFIG_MANAGER_FOR_DRIVER_MODULE_NAME);
   return B_OK;
}


uchar mask_to_value(uint32 mask)
{
   uchar value;
   if (!mask) return 0;
   for (value=0,mask>>=1;mask;value++,mask>>=1)
      ;
   return value;
}

static int dump_mask(uint32 mask, uint16 *p_Irq)
{
   uint16 i = 0;
   if (!mask)
      return(-1);

   for (;mask;mask>>=1,i++)
   {
      if (mask & 1) 
      {
         *p_Irq = i;
         return(0);
      }
   }
   return(-1);
}

static void dump_device_info(struct device_info *info, 
                              struct device_configuration *current, 
                              TS_SERIAL_device *pas_Devices)
{
   resource_descriptor r;
   TS_SERIAL_device  *p_CurDevice;

   /* if serial device */
   if ((info->devtype.base == 7) && (info->devtype.subtype == 0)) 
   {
      DBG_PRINT("dump_device_info : Found serial device");
      /* Use the device only if it is configured */
      if (info->flags & B_DEVICE_INFO_CONFIGURED) 
      {
         DBG_PRINT("dump_device_info : Found configured device");
         /* Assume we have 1 irq and 1 port ressource per serial port */
         /* alloc new descriptor */
         p_CurDevice = SERIAL_NewDevice(pas_Devices);
         if(p_CurDevice == NULL)
         {
            DBG_PRINT("No free descriptor found");
            return;
         }

         /* fill descriptor */
         gps_ConfigManager->get_nth_resource_descriptor_of_type(
               current, 0, B_IO_PORT_RESOURCE, 
               &r, sizeof(resource_descriptor));
         p_CurDevice->base_addr = r.d.r.minbase;

         gps_ConfigManager->get_nth_resource_descriptor_of_type(
               current, 0, B_IRQ_RESOURCE, &r, 
               sizeof(resource_descriptor));
         if(dump_mask(r.d.m.mask, &p_CurDevice->irq) != 0)
         {
            DBG_PRINT("Error getting Irq infos");
            SERIAL_FreeDevice(p_CurDevice);
            return;
         }

         dprintf("BERC - SERIAL : dev addr = 0x%X, irq = %d\n",
                  p_CurDevice->base_addr, p_CurDevice->irq);
      } 
   }
}

int detect(TS_SERIAL_device *pas_Devices)
{
   status_t size;
   uint64 cookie;
   void *current;
   struct device_info dummy, *info;
   bus_type bus;

   bus = B_ISA_BUS;

   if( init_cm_wrapper() != B_OK) 
   {
      DBG_PRINT("Error initializing configuration manager\n");
      return(-1);
   }

   /* build device list */
   cookie = 0;
   while (gps_ConfigManager->get_next_device_info(bus, &cookie, &dummy, sizeof(dummy)) == B_OK) 
   {
      info = (struct device_info *)malloc(dummy.size);
      if (!info) 
      {
         DBG_PRINT("Out of core\n");
         continue;
      }
      
      if (gps_ConfigManager->get_device_info_for(cookie, info, dummy.size) != B_OK) 
      {
         DBG_PRINT("Error getting device information\n");
         free(info);
         continue;
      }

      size = gps_ConfigManager->get_size_of_current_configuration_for(cookie);
      if (size < B_OK) 
      {
         DBG_PRINT("Error getting the size of the current configuration\n");
         free(info);
         continue;
      }
      current = malloc(size);
      if (!current) 
      {
         DBG_PRINT("Out of core\n");
         free(info);
         continue;
      }

      if (gps_ConfigManager->get_current_configuration_for(cookie, current, size) != B_OK) 
      {
         DBG_PRINT("Error getting the current configuration\n");
         free(info);
         free(current);
         continue;
      }

      dump_device_info(info, current, pas_Devices);
      free(info);
      free(current);
   }
   uninit_cm_wrapper();

   return(0);
}
